% Pole Placement Design for Magnetic Suspension
clear all;
% Vehicle Model Parameters
K_IP = 7.7;
T_IP = 0.3;
K_VD = 0.17;
T_VD = 19.5;
K_d = 11.5;

% =============================================
% Youla Design for Magnetic Suspension
% =============================================
% Bearing Model Parameters
Km = 1E-4;
d = 0.04;
m = 0.1;
g = 10;
i0 = sqrt(m*g/Km)*d/2;
Ki = 2*Km*i0/(d/2)^2;
Kx = 2*Km*i0^2/(d/2)^3;


% =================
% Youla deaign
% =================
% Inner Loop
tau = sqrt(m/Kx);
T2 = 1/3;
K_R = (tau^2/3 + 1)*Kx/Ki;
T1 = (3 + 1/tau^2)/(1 + 3/tau^2);
% New plant transfer function
s = tf('s');
T_tilde = (1 + s*T1)*K_R/tau^2/T2/Kx/(s^3 + 3*s^2 + 3*s + 1);
% Desired closed loop
T = 1/(1 + 0.1*s)^2;
% Q - parameter
Q = T/T_tilde;
% Controller
C = minreal(Q/(1 - T_tilde*Q));
% reference tracking
ref = 1;
d_load = 0;
% simulation
options = simset('Solver','ode23s');
sim('ECE441_Ex5_solution_11',[0,5],options);
figure(1);
plot(time,y);
legend('reference step $x = d/4$')
grid
xlabel('time [sec]')
ylabel('potition [m/s]')
% Discturbancestep
ref = 0;
d_load = 0.05;
sim('ECE441_Ex5_solution_11',[0,5],options);
figure(2);
plot(time,y);
legend('disturbance step $F_\mathrm{L} = 0.5 N$','Interpreter','latex')
grid
xlabel('time [sec]')
ylabel('position [m/s]')

% ====================================
% Feedback Loop with time delay
% reference tracking
options = simset('Solver','ode23s');
ref = 1;
d_load = 0;
smith_on = 0;
sim('ECE441_Ex5_solution_12',[0,5],options);
figure(3);
plot(time,y);
legend('reference step $x = d/4$')
grid
xlabel('time [sec]')
ylabel('potition [m/s]')
% Discturbancestep
ref = 0;
d_load = 0.05;
sim('ECE441_Ex5_solution_12',[0,5],options);
figure(4);
plot(time,y);
legend('disturbance step $F_\mathrm{L} = 0.5 N$','Interpreter','latex')
grid
xlabel('time [sec]')
ylabel('position [m/s]')


% ====================================
% Smith Predictor
% reference tracking
options = simset('Solver','ode23s');
ref = 1;
d_load = 0;
smith_on = 1;
sim('ECE441_Ex5_solution_12',[0,5],options);
figure(5);
plot(time,y);
legend('reference step $x = d/4$')
grid
xlabel('time [sec]')
ylabel('potition [m/s]')
% Discturbancestep
ref = 0;
d_load = 0.05;
sim('ECE441_Ex5_solution_12',[0,5],options);
figure(6);
plot(time,y);
legend('disturbance step $F_\mathrm{L} = 0.5 N$','Interpreter','latex')
grid
xlabel('time [sec]')
ylabel('position [m/s]')

